@@ -1,20 +1,21 @@
+from __future__ import print_function
+
 import os
 import sys
 import time
 import binascii
 import hashlib
 import unicodedata
-import mapping
 import json
 import getpass
 
-import tools
-import messages_pb2 as proto
-import types_pb2 as types
-import protobuf_json
-from trezorlib.debuglink import DebugLink
 from mnemonic import Mnemonic
 
+from . import tools
+from . import mapping
+from . import messages_pb2 as proto
+from . import types_pb2 as types
+from .debuglink import DebugLink
 
 # try:
 #     from PIL import Image
@@ -88,15 +89,18 @@ class expect(object):
         return wrapped_f
 
 def normalize_nfc(txt):
-    # Normalize string to UTF8 NFC for sign_message
-    if isinstance(txt, str):
-        utxt = txt.decode('utf8')
-    elif isinstance(txt, unicode):
-        utxt = txt
+    if sys.version_info[0] < 3:
+        if isinstance(txt, unicode):
+            return unicodedata.normalize('NFC', txt).encode('utf-8')
+        if isinstance(txt, str):
+            return unicodedata.normalize('NFC', txt.decode('utf-8')).encode('utf-8')
     else:
-        raise Exception("String value expected")
+        if isinstance(txt, bytes):
+            return unicodedata.normalize('NFC', txt.decode('utf-8')).encode('utf-8')
+        if isinstance(txt, str):
+            return unicodedata.normalize('NFC', txt).encode('utf-8')
 
-    return unicodedata.normalize('NFC', utxt)
+    raise Exception('unicode/str or bytes/str expected')
 
 class BaseClient(object):
     # Implements very basic layer of sending raw protobuf
@@ -105,6 +109,9 @@ class BaseClient(object):
         self.transport = transport
         super(BaseClient, self).__init__()  # *args, **kwargs)
 
+    def cancel(self):
+        self.transport.write(proto.Cancel())
+
     def call_raw(self, msg):
         try:
             self.transport.session_begin()
@@ -189,7 +196,7 @@ class TextUIMixin(object):
         passphrase = getpass.getpass('')
         log("Confirm your Passphrase: ")
         if passphrase == getpass.getpass(''):
-            passphrase = unicode(str(bytearray(passphrase, 'utf-8')), 'utf-8')
+            passphrase = normalize_nfc(passphrase)
             return proto.PassphraseAck(passphrase=passphrase)
         else:
             log("Passphrase did not match! ")
@@ -197,7 +204,10 @@ class TextUIMixin(object):
 
     def callback_WordRequest(self, msg):
         log("Enter one word of mnemonic: ")
-        word = raw_input()
+        try:
+            word = raw_input()
+        except NameError:
+            word = input() # Python 3
         return proto.WordAck(word=word)
 
 class DebugLinkMixin(object):
@@ -270,10 +280,10 @@ class DebugLinkMixin(object):
         self.pin_correct = pin_correct
 
     def set_passphrase(self, passphrase):
-        self.passphrase = unicode(str(bytearray(Mnemonic.normalize_string(passphrase), 'utf-8')), 'utf-8')
+        self.passphrase = normalize_nfc(passphrase)
 
     def set_mnemonic(self, mnemonic):
-        self.mnemonic = unicode(str(bytearray(Mnemonic.normalize_string(mnemonic), 'utf-8')), 'utf-8').split(' ')
+        self.mnemonic = normalize_nfc(passphrase)
 
     def call_raw(self, msg):
 
@@ -365,7 +375,8 @@ class ProtocolMixin(object):
         # Convert minus signs to uint32 with flag
         return [ int(abs(x) | self.PRIME_DERIVATION_FLAG) if x < 0 else x for x in n ]
 
-    def expand_path(self, n):
+    @staticmethod
+    def expand_path(n):
         # Convert string of bip32 path to list of uint32 integers with prime flags
         # 0/-1/1' -> [0, 0x80000001, 0x80000001]
         if not n:
@@ -384,7 +395,7 @@ class ProtocolMixin(object):
             x = abs(int(x))
 
             if prime:
-                x |= self.PRIME_DERIVATION_FLAG
+                x |= ProtocolMixin.PRIME_DERIVATION_FLAG
 
             path.append(x)
 
@@ -406,6 +417,40 @@ class ProtocolMixin(object):
         else:
             return self.call(proto.GetAddress(address_n=n, coin_name=coin_name, show_display=show_display))
 
+    @field('address')
+    @expect(proto.EthereumAddress)
+    def ethereum_get_address(self, n, show_display=False, multisig=None):
+        n = self._convert_prime(n)
+        return self.call(proto.EthereumGetAddress(address_n=n, show_display=show_display))
+
+    def ethereum_sign_tx(self, n, nonce, gas_price, gas_limit, to, value, data=''):
+        from rlp.utils import int_to_big_endian
+
+        n = self._convert_prime(n)
+
+        try:
+            self.transport.session_begin()
+
+            response = self.call(proto.EthereumSignTx(
+                address_n=n,
+                nonce=int_to_big_endian(nonce),
+                gas_price=int_to_big_endian(gas_price),
+                gas_limit=int_to_big_endian(gas_limit),
+                to=to,
+                value=int_to_big_endian(value)))
+            if data:
+                data, chunk = data[1024:], data[:1024]
+                response.data_initial_chunk = chunk
+                response.data_length = len(data)
+
+            while response.HasField('data_length'):
+                data, chunk = data[1024:], data[:1024]
+                response = self.call(proto.EthereumTxAck(data_chunk=chunk))
+
+            return response.signature_v, response.signature_r, response.signature_s
+        finally:
+            self.transport.session_end()
+
     @field('entropy')
     @expect(proto.Entropy)
     def get_entropy(self, size):
@@ -455,30 +500,21 @@ class ProtocolMixin(object):
     @expect(proto.MessageSignature)
     def sign_message(self, coin_name, n, message):
         n = self._convert_prime(n)
-
-        try:
-            # Convert message to UTF8 NFC (seems to be a bitcoin-qt standard)
-            message = normalize_nfc(message)
-            # Convert message to ASCII stream
-            message = str(bytearray(message, 'utf-8'))
-        except:
-            pass # it was not UTF8 string
-
+        # Convert message to UTF8 NFC (seems to be a bitcoin-qt standard)
+        message = normalize_nfc(message)
         return self.call(proto.SignMessage(coin_name=coin_name, address_n=n, message=message))
 
     @expect(proto.SignedIdentity)
     def sign_identity(self, identity, challenge_hidden, challenge_visual, ecdsa_curve_name=DEFAULT_CURVE):
         return self.call(proto.SignIdentity(identity=identity, challenge_hidden=challenge_hidden, challenge_visual=challenge_visual, ecdsa_curve_name=ecdsa_curve_name))
 
-    def verify_message(self, address, signature, message):
-        try:
-            # Convert message to UTF8 NFC (seems to be a bitcoin-qt standard)
-            message = normalize_nfc(message)
-        # Convert message to ASCII stream
-            message = str(bytearray(message, 'utf-8'))
-        except:
-            pass # it was not UTF8 string
+    @expect(proto.ECDHSessionKey)
+    def get_ecdh_session_key(self, identity, peer_public_key, ecdsa_curve_name=DEFAULT_CURVE):
+        return self.call(proto.GetECDHSessionKey(identity=identity, peer_public_key=peer_public_key, ecdsa_curve_name=ecdsa_curve_name))
 
+    def verify_message(self, address, signature, message):
+        # Convert message to UTF8 NFC (seems to be a bitcoin-qt standard)
+        message = normalize_nfc(message)
         try:
             if address:
                 resp = self.call(proto.VerifyMessage(address=address, signature=signature, message=message))
@@ -733,7 +769,7 @@ class ProtocolMixin(object):
         mnemonic = Mnemonic.normalize_string(mnemonic)
 
         # Convert mnemonic to ASCII stream
-        mnemonic = unicode(str(bytearray(mnemonic, 'utf-8')), 'utf-8')
+        mnemonic = normalize_nfc(mnemonic)
 
         if self.features.initialized:
             raise Exception("Device is initialized already. Call wipe_device() and try again.")
@@ -759,7 +795,7 @@ class ProtocolMixin(object):
             raise Exception("Invalid length of xprv")
 
         node = types.HDNodeType()
-        data = tools.b58decode(xprv, None).encode('hex')
+        data = binascii.hexlify(tools.b58decode(xprv, None))
 
         if data[90:92] != '00':
             raise Exception("Contain invalid private key")
