@@ -2,25 +2,24 @@
 
 import hid
 import time
-import platform
-from transport import Transport, ConnectionError, NotImplementedException
+from .transport import Transport, ConnectionError
 
 DEVICE_IDS = [
-#    (0x10c4, 0xea80),  # Shield
-    (0x534c, 0x0001),  # Trezor
+    (0x534c, 0x0001),  # TREZOR
 ]
 
 class FakeRead(object):
     # Let's pretend we have a file-like interface
     def __init__(self, func):
         self.func = func
-        
+
     def read(self, size):
         return self.func(size)
 
 class HidTransport(Transport):
     def __init__(self, device, *args, **kwargs):
         self.hid = None
+        self.hid_version = None
         self.buffer = ''
         # self.read_timeout = kwargs.get('read_timeout')
         device = device[int(bool(kwargs.get('debug_link')))]
@@ -54,7 +53,7 @@ class HidTransport(Transport):
                     raise Exception("Unknown USB interface number: %d" % interface_number)
 
         # List of two-tuples (path_normal, path_debuglink)
-        return devices.values()
+        return list(devices.values())
 
     def is_connected(self):
         """
@@ -64,35 +63,44 @@ class HidTransport(Transport):
             if d['path'] == self.device:
                 return True
         return False
-        
+
     def _open(self):
-        self.buffer = ''
+        self.buffer = bytearray()
         self.hid = hid.device()
         self.hid.open_path(self.device)
         self.hid.set_nonblocking(True)
-        # the following was needed just for Trezor Shield
-        # self.hid.send_feature_report([0x41, 0x01]) # enable UART
-        # self.hid.send_feature_report([0x43, 0x03]) # purge TX/RX FIFOs
-    
+        # determine hid_version
+        r = self.hid.write([0, 63, ] + [0xFF] * 63)
+        if r == 65:
+            self.hid_version = 2
+            return
+        r = self.hid.write([63, ] + [0xFF] * 63)
+        if r == 64:
+            self.hid_version = 1
+            return
+        raise ConnectionError("Unknown HID version")
+
     def _close(self):
         self.hid.close()
-        self.buffer = ''
+        self.buffer = bytearray()
         self.hid = None
-    
+
     def ready_to_read(self):
         return False
-    
+
     def _write(self, msg, protobuf_msg):
         msg = bytearray(msg)
-        while len(msg):            
-            # Report ID, data padded to 63 bytes
-            self.hid.write([63, ] + list(msg[:63]) + [0] * (63 - len(msg[:63])))
+        while len(msg):
+            if self.hid_version == 2:
+                self.hid.write([0, 63, ] + list(msg[:63]) + [0] * (63 - len(msg[:63])))
+            else:
+                self.hid.write([63, ] + list(msg[:63]) + [0] * (63 - len(msg[:63])))
             msg = msg[63:]
-            
+
     def _read(self):
         (msg_type, datalen) = self._read_headers(FakeRead(self._raw_read))
         return (msg_type, self._raw_read(datalen))
-                    
+
     def _raw_read(self, length):
         start = time.time()
         while len(self.buffer) < length:
@@ -111,14 +119,14 @@ class HidTransport(Transport):
                 continue
 
             report_id = data[0]
-            
+
             if report_id > 63:
                 # Command report
                 raise Exception("Not implemented")
-                                     
+
             # Payload received, skip the report ID
-            self.buffer += str(bytearray(data[1:]))
+            self.buffer.extend(bytearray(data[1:]))
 
         ret = self.buffer[:length]
         self.buffer = self.buffer[length:]
-        return ret
+        return bytes(ret)
